import 'package:args/command_runner.dart';
import 'package:stacked_tools/src/locator.dart';
import 'package:stacked_tools/src/message_constants.dart';
import 'package:stacked_tools/src/mixins/project_structure_validator_mixin.dart';
import 'package:stacked_tools/src/services/pubspec_service.dart';
import 'package:stacked_tools/src/services/template_service.dart';
import 'package:stacked_tools/src/templates/template_constants.dart';

class CreateViewCommand extends Command with ProjectStructureValidator {
  final _templateService = locator<TemplateService>();
  final _pubspecService = locator<PubspecService>();

  @override
  String get description =>
      'Creates a view with all associated files and makes necessary code changes for adding a view.';

  @override
  String get name => kTemplateNameView;

  CreateViewCommand() {
    // TODO: Move the command names and flags into a constants file to make it easier to work with
    argParser.addFlag(
      'exclude-route',
      defaultsTo: false,
      help: kCommandHelpExcludeRoute,
    );
  }

  @override
  Future<void> run() async {
    await _pubspecService.initialise();

    final outputPath = argResults!.rest.length > 1 ? argResults!.rest[1] : null;
    await validateStructure(outputPath: outputPath);

    await _templateService.renderTemplate(
      templateName: kTemplateNameView,
      name: argResults!.rest.first,
      outputPath: outputPath,
      verbose: true,
      excludeRoute: argResults!['exclude-route'],
    );
  }
}
